// -----------------------------------------------------------------------------
// Copyright 2009-2012 Patrick Näf (herzbube@herzbube.ch)
//
// This file is part of the dgsmonX project.
//
// dgsmonX is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// dgsmonX is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with dgsmonX. If not, see <http://www.gnu.org/licenses/>.
// -----------------------------------------------------------------------------


// Forward declarations
@class DGSMonXServerMenuItem;


// -----------------------------------------------------------------------------
/// @class DGSMonXServer
///
/// @brief The DGSMonXServer class represents a Dragon Go Server installation
/// located somewhere on the network.
///
/// DGSMonXServer stores a number of attributes that are obtained from the
/// User Defaults system and can be changed in the application's preferences
/// dialog.
///
/// DGSMonXServer also stores the results of the most recent game state check
/// performed on it. These attributes are lost when the application is shut
/// down.
///
/// Last but not least, DGSMonXServer holds a reference to a menu item that
/// is used to represent the server in the menu associated with the
/// application's status item.
///
///
/// @par Unique IDs
///
/// The unique ID of an DGSMonXServer object is used to identify the object
/// across the "Distributed Objects" boundary that exists between threads in
/// the dgsmonX project.
///
/// Because dgsmonX uses the "Distributed Objects" system to communicate between
/// threads, it is not possible to pass the pointer to a DGSMonXServer object
/// from one thread to the other, and then use that pointer (i.e. the memory
/// address of the object) to identify the object. The reason: In the receiving
/// thread the DGSMonXServer object becomes a proxy object (an instance of
/// NSDistantObject) whose memory address is different from the original
/// DGSMonXServer.
// -----------------------------------------------------------------------------
@interface DGSMonXServer : NSObject
{
@private
  /// @name Attributes from User Defaults
  //@{
  bool m_monitored;
  NSString* m_name;
  int m_protocol;
  NSString* m_url;
  NSString* m_userID;
  NSString* m_userPassword;
  //@}

  /// @name Game state check results
  //@{
  int m_checkState;
  bool m_checkSucceeded;
  NSString* m_errorDetails;
  int m_numberOfGames;
  //@}

  /// @brief The server's unique ID during runtime.
  int m_uniqueID;
  /// @brief The menu item used to represent the server.
  DGSMonXServerMenuItem* m_menuItem;
}

/// @name Initializers
//@{
- (id) init;
- (id) initWithDictionary:(NSDictionary*)dictionary;
//@}

/// @name Unique ID handling
//@{
- (int) uniqueID;
//@}

/// @name Accessors for User Defaults attributes
//@{
- (bool) monitored;
- (void) setMonitored:(bool)monitored;
- (NSString*) name;
- (void) setName:(NSString*)name;
- (int) protocol;
- (void) setProtocol:(int)protocol;
- (NSString*) url;
- (void) setUrl:(NSString*)url;
- (NSString*) userID;
- (void) setUserID:(NSString*)userID;
- (NSString*) userPassword;
- (void) setUserPassword:(NSString*)userPassword;
- (NSDictionary*) asDictionary;
//@}

/// @name Accessors for game state check results
//@{
- (int) checkState;
- (void) setCheckState:(int)checkState;
- (bool) checkSucceeded;
- (void) setCheckSucceeded:(bool)checkSucceeded;
- (NSString*) errorDetails;
- (void) setErrorDetails:(NSString*)errorDetails;
- (int) numberOfGames;
- (void) setNumberOfGames:(int)numberOfGames;
- (bool) itsYourTurn;
//@}

/// @name Menu item accessors
//@{
- (DGSMonXServerMenuItem*) menuItem;
//@}

/// @name Other methods
//@{
- (NSString*) baseURL;
- (int) testConnection;
//@}

@end
