// -----------------------------------------------------------------------------
// Copyright 2009-2012 Patrick Näf (herzbube@herzbube.ch)
//
// This file is part of the dgsmonX project.
//
// dgsmonX is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// dgsmonX is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with dgsmonX. If not, see <http://www.gnu.org/licenses/>.
// -----------------------------------------------------------------------------


// Project includes
#import "DGSMonXServerLogin.h"
#import "DGSMonXServer.h"
#import "DGSMonXUtilities.h"


/// @brief This category declares private methods for the DGSMonXServerLogin
/// class.
@interface DGSMonXServerLogin()
/// @name Initialization and deallocation
//@{
- (void) dealloc;
//@}
/// @name NSURLConnectionDelegate protocol methods
//@{
- (void) connection:(NSURLConnection*)connection didReceiveResponse:(NSURLResponse*)response;
- (void) connection:(NSURLConnection*)connection didReceiveData:(NSData*)data;
- (void) connection:(NSURLConnection*)connection didFailWithError:(NSError*)error;
- (void) connectionDidFinishLoading:(NSURLConnection*)connection;
//@}
/// @name Helpers
//@{
- (void) login;
- (void) loginSucceeded;
- (void) loginFailed:(NSString*)errorDetails;
//@}
/// @name Privately declared properties
//@{
/// @brief The most recent connection used for an HTTP protocol exchange.
@property(nonatomic, retain) NSURLConnection* connection;
/// @brief The most recent response received during an HTTP protocol exchange.
@property(nonatomic, retain) NSURLResponse* receivedResponse;
/// @brief Container that holds data received during an HTTP protocol
/// exchange.
@property(nonatomic, retain) NSMutableData* receivedData;
//@}
@end


@implementation DGSMonXServerLogin

@synthesize server;
@synthesize delegate;
@synthesize connection;
@synthesize receivedResponse;
@synthesize receivedData;


// -----------------------------------------------------------------------------
/// @brief Initializes a DGSMonXServerLogin object that performs a login on
/// @a aServer and calls back @a aDelegate when the login is done.
///
/// @a aServer and @a aDelegate must not be @e nil.
///
/// @note This is the designated initializer of DGSMonXServerLogin.
// -----------------------------------------------------------------------------
- (id) initWithServer:(DGSMonXServer*)aServer delegate:(id<DGSMonXServerLoginDelegate>)aDelegate;
{
  // Call designated initializer of superclass (NSObject)
  self = [super init];
  if (! self)
    return nil;

  self.server = aServer;
  self.delegate = aDelegate;

  // This is assigned when the game state check begins
  self.connection = nil;
  // This is assigned/retained/released during a HTTP protocol exchange
  self.receivedResponse = nil;
  self.receivedData = [NSMutableData data];

  // Initiate the login process
  [self login];

  return self;
}

// -----------------------------------------------------------------------------
/// @brief Deallocates memory allocated by this DGSMonXServer object.
// -----------------------------------------------------------------------------
- (void) dealloc
{
  self.connection = nil;
  self.receivedResponse = nil;
  self.receivedData = nil;
  // Not strictly necessary, these are not retained
  self.server = nil;
  self.delegate = nil;
  [super dealloc];
}

// -----------------------------------------------------------------------------
/// @brief Initiates the login process.
// -----------------------------------------------------------------------------
- (void) login
{
  // Construct request for login page
  NSString* loginPageWithQueryString = [NSString stringWithFormat:@"%@?userid=%@&passwd=%@",
                                        loginPage,
                                        [self.server userID],
                                        [self.server userPassword]];
  NSURLRequest* loginPageRequest = [DGSMonXUtilities request:self.server forPage:loginPageWithQueryString];
  if (! loginPageRequest)
  {
    [self loginFailed:[NSString stringWithFormat:@"Failed to construct HTTP request for login page"]];
    return;
  }

  // Create the connection that retrieves the login page. The connection
  // immediately starts loading the data.
  self.connection = [[NSURLConnection alloc] initWithRequest:loginPageRequest delegate:self];
  if (! self.connection)
  {
    [self loginFailed:[NSString stringWithFormat:@"Connection for login page could not be initialized"]];
    return;
  }
}

// -----------------------------------------------------------------------------
/// @brief Sent when @a connection has received sufficient data from the server
/// to construct @a response.
///
/// This implementation does the following things:
/// - initialize a data structure that will later be used by
///   connection:didReceiveData:() to store content data that is incrementally
///   received from the server
/// - store @a response so that connectionDidFinishLoading:() can evaluate
///   which page the data came from
///
/// @note This is an NSURLConnectionDelegate protocol method.
// -----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didReceiveResponse:(NSURLResponse*)response
{
  if (self.receivedResponse)
    [self.receivedResponse autorelease];
  self.receivedResponse = [response retain];
  [self.receivedData setLength:0];
}

// -----------------------------------------------------------------------------
/// @brief Sent as @a connection loads data incrementally.
///
/// This implementation appends @a data to the data structure previously
/// initialized by connection:didReceiveResponse:().
///
/// @note This is an NSURLConnectionDelegate protocol method.
// -----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didReceiveData:(NSData*)data
{
  [self.receivedData appendData:data];
}

// -----------------------------------------------------------------------------
/// @brief Sent when @a connection fails to load its request successfully.
///
/// This implementaton terminates the login process by invoking loginFailed:().
/// The error details are taken from the message encapsulated by @a error.
///
/// @note This happens for instance if there is no network connection to the
/// server (e.g. no internet connection, or the server name is wrong)
///
/// @note This is an NSURLConnectionDelegate protocol method.
// -----------------------------------------------------------------------------
- (void) connection:(NSURLConnection*)connection didFailWithError:(NSError*)error
{
  [self loginFailed:[error localizedDescription]];
}

// -----------------------------------------------------------------------------
/// @brief Sent when @a connection has finished loading successfully.
///
/// This implementaton checks the NSURLResponse object that was cached by
/// connection:didReceiveResponse:() in order to determine what should be done
/// next. The NSURLResponse may refer to the following pages:
/// - error.php: It is assumed that login failed. The login process is
///   terminated by invoking loginFailed:(). The error details are taken from
///   the query string that has been supplied to error.php.
/// - Anything else: It is assumed that login succeeded. The login process is
///   concluded by invoking loginSucceeded().
///
/// @note This is an NSURLConnectionDelegate protocol method.
// -----------------------------------------------------------------------------
- (void) connectionDidFinishLoading:(NSURLConnection*)connection
{
  int statusCode = [(NSHTTPURLResponse*)self.receivedResponse statusCode];
  if (statusCode >= 400 && statusCode <= 599)
  {
    [self loginFailed:[NSString stringWithFormat:@"HTTP status code %d", statusCode]];
    return;
  }

  NSURL* responseURL = [self.receivedResponse URL];
  NSString* pageName = (NSString*)[[[responseURL path] componentsSeparatedByString:@"/"] lastObject];
  if (NSOrderedSame == [pageName compare:errorPage])
  {
    NSString* errorDetails = [DGSMonXUtilities errorDetailsFromErrorPageURL:responseURL];
    if (! errorDetails)
      errorDetails = @"Login failed for unknown reason";
    [self loginFailed:errorDetails];
  }
  else
  {
    [self loginSucceeded];
  }
}

// -----------------------------------------------------------------------------
/// @brief Notifies the delegate that the login process finished successfully.
// -----------------------------------------------------------------------------
- (void) loginSucceeded
{
  [self.delegate loginSucceeded:self];
}

// -----------------------------------------------------------------------------
/// @brief Notifies the delegate that the login process finished with failure.
// -----------------------------------------------------------------------------
- (void) loginFailed:(NSString*)errorDetails
{
  [self.delegate loginFailed:self errorDetails:errorDetails];
}

// -----------------------------------------------------------------------------
/// @brief Terminates the login process and notifies the delegate that login has
/// been cancelled.
// -----------------------------------------------------------------------------
- (void) cancel
{
  [self.connection cancel];   // we should not get any more delegate messages
  [self.delegate loginCancelled:self];
}

@end
