// -----------------------------------------------------------------------------
// Copyright 2009-2012 Patrick Näf (herzbube@herzbube.ch)
//
// This file is part of the dgsmonX project.
//
// dgsmonX is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// dgsmonX is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with dgsmonX. If not, see <http://www.gnu.org/licenses/>.
// -----------------------------------------------------------------------------


// Project includes
#import "DGSMonXServerMenuItem.h"
#import "DGSMonXServer.h"


/// @brief This category declares private methods for the DGSMonXServerMenuItem
/// class.
@interface DGSMonXServerMenuItem()
/// @name Initialization and deallocation
//@{
- (void) dealloc;
//@}
@end


@implementation DGSMonXServerMenuItem

// -----------------------------------------------------------------------------
/// @brief Initializes a DGSMonXServerMenuItem object that represents
/// @a server.
///
/// @note This is the designated initializer of DGSMonXServerMenuItem.
// -----------------------------------------------------------------------------
- (id) initWithServer:(DGSMonXServer*)server
{
  // Call designated initializer of superclass (NSObject)
  self = [super init];
  if (! self)
    return nil;

  // Don't retain to prevent a retain cycle.
  // Note: Initially I tried to use NSMenuItem's setRepresentedObject:() method
  // to reference the server (this would have saved me a member and a getter
  // method), but unfortunately that method *does* retain.
  m_server = server;

  return self;
}

// -----------------------------------------------------------------------------
/// @brief Deallocates memory allocated by this DGSMonXServerMenuItem object.
// -----------------------------------------------------------------------------
- (void) dealloc
{
  [super dealloc];
}

// -----------------------------------------------------------------------------
/// @brief Returns the DGSMonXServer object represented by this
/// DGSMonXServerMenuItem.
///
/// @attention When a server is deleted in the Preferences dialog, the server's
/// DGSMonXServer object is deallocated, while this menu item goes on living
/// because it is still retained by the menu associated with the application's
/// status item. This method will therefore return an invalid DGSMonXServer
/// object for some time. It is expected that DGSMonXController, being the sole
/// user of DGSMonXServerMenuItem, properly manages this issue.
// -----------------------------------------------------------------------------
- (DGSMonXServer*) server
{
  return m_server;
}

// -----------------------------------------------------------------------------
/// @brief Updates the visual representation of this DGSMonXServerMenuItem.
///
/// @attention This method must not be invoked @b after the server represented
/// by this DGSMonXServerMenuItem has been deleted in the Preferences dialog.
// -----------------------------------------------------------------------------
- (void) update
{
  // May return invalid object! See server() method docs for details why this
  // should not be a problem.
  DGSMonXServer* server = [self server];
  NSString* state;
  if (! [server monitored])
    state = @"Monitoring disabled";
  else
  {
    switch ([server checkState])
    {
      case NonePerformedCheckState:
        state = @"No check performed yet";
        break;
      case InProgressCheckState:
        state = @"Check in progress...";
        break;
      case DidFinishCheckState:
        if ([server checkSucceeded])
        {
          int numberOfGames = [server numberOfGames];
          if (0 == numberOfGames)
            state = @"No games waiting";
          else
            state = [NSString stringWithFormat:@"%d games waiting", numberOfGames];
        }
        else
        {
          NSString* errorDetails = [server errorDetails];
          if (! errorDetails)
            state = @"Check failed, no error details";
          else
            state = [NSString stringWithFormat:@"Check failed, error details: %@", errorDetails];
        }
        break;
      case WasCancelledCheckState:
        state = @"Check was cancelled";
        break;
      default:
        state = @"Unknown state";
        break;
    }
  }
  [self setTitle:[NSString stringWithFormat:@"%@: %@", [server name], state]];
}

@end
